@@ -5,9 +5,11 @@ from timeit import default_timer as timer
 from bakplane.bakplane_pb2 import ResolveResourcePathResponse, ResolveResourceIntentResponse, ComparisonType
 from bakplane.extensions.base import WriteResponse, WriteRequest, ReadResponse, ReadAllRequest, ReadRequest, \
     ReadRequestBuildingBlocks
-from bakplane.extensions.spark.utils import to_spark_save_mode, timestamp_to_date_string
+from bakplane.extensions.spark.utils import to_spark_save_mode, timestamp_to_date_string, datetime_to_date_string
 from bakplane.utils import to_execution_statistics
 
+from datetime import datetime
+
 
 def __build_read_all_query(r: ReadAllRequest, res: ResolveResourcePathResponse) -> str:
     return f'select {", ".join(r.columns)} from {res.context["schema_name"]}.{res.context["table_name"]}'
@@ -62,32 +64,47 @@ def __build_read_query(r: ReadRequest, res: ResolveResourceIntentResponse, diale
         asset_table=a,
         selectable=m.join(a, and_(*clauses)),
         effective_start_dt=effective_start_dt,
-        effective_end_dt=effective_end_dt
+        effective_end_dt=effective_end_dt,
+        knowledge_start_dt=r.knowledge_start_dt,
+        knowledge_end_dt=r.knowledge_end_dt
     )
 
     if r.query_builder:
         res = r.query_builder(blocks)
     else:
+
+        constraints = []
+
         if r.additional_constraints is not None:
-            constraints = and_(
-                a.c.effective_start_dt <= m.c.effective_end_dt,
-                a.c.effective_end_dt >= m.c.effective_start_dt,
-                a.c.effective_start_dt <= timestamp_to_date_string(res.effective_dating.end_dt.seconds),
-                a.c.effective_end_dt >= timestamp_to_date_string(res.effective_dating.start_dt.seconds),
-                r.additional_constraints(blocks)
-            )
+            constraints.append(r.additional_constraints)
+
+        if r.effective_dating_hint is None:
+            constraints.append(a.c.effective_start_dt <= m.c.effective_end_dt)
+            constraints.append(a.c.effective_end_dt >= m.c.effective_start_dt)
+            constraints.append(a.c.effective_start_dt <= timestamp_to_date_string(res.effective_dating.end_dt.seconds))
+            constraints.append(a.c.effective_end_dt >= timestamp_to_date_string(res.effective_dating.start_dt.seconds))
         else:
-            constraints = and_(
-                a.c.effective_start_dt <= m.c.effective_end_dt,
-                a.c.effective_end_dt >= m.c.effective_start_dt,
-                a.c.effective_start_dt <= timestamp_to_date_string(res.effective_dating.end_dt.seconds),
-                a.c.effective_end_dt >= timestamp_to_date_string(res.effective_dating.start_dt.seconds),
-            )
+            if isinstance(r.effective_dating_hint, tuple):
+                s = datetime_to_date_string(typing.cast(datetime, r.effective_dating_hint[0]))
+                e = datetime_to_date_string(typing.cast(datetime, r.effective_dating_hint[1]))
+
+                constraints.append(a.c.effective_start_dt <= e)
+                constraints.append(a.c.effective_end_dt >= s)
+            elif isinstance(r.effective_dating_hint, str):
+                col = typing.cast(str, r.effective_dating_hint)
+
+                s = col + '_start_dt'
+                e = col + '_end_dt'
+
+                constraints.append(a.c.effective_start_dt <= m.c.get(e))
+                constraints.append(a.c.effective_end_dt >= m.c.get(s))
+            else:
+                raise RuntimeError('Invalid type for effective_dating_hint.')
 
         res = (
             select([a.c.get(x) for x in r.columns])
                 .select_from(m.join(a, and_(*clauses)))
-                .where(constraints)
+                .where(and_(*constraints))
         )
 
     query = str(
